package dev.shadowsoffire.apotheosis.ench.table;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.ench.EnchModule;
import dev.shadowsoffire.apotheosis.ench.api.IEnchantingBlock;
import dev.shadowsoffire.apotheosis.ench.table.EnchantingStatRegistry;
import dev.shadowsoffire.placebo.reload.DynamicRegistry;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraftforge.common.crafting.conditions.ICondition;

public class EnchantingStatRegistry extends DynamicRegistry<EnchantingStatRegistry.BlockStats> {
  public static final Gson GSON = (new GsonBuilder()).setPrettyPrinting().create();
  
  public static final EnchantingStatRegistry INSTANCE = new EnchantingStatRegistry();
  
  private final Map<Block, Stats> statsPerBlock = new HashMap<>();
  
  private float absoluteMaxEterna = 50.0F;
  
  protected EnchantingStatRegistry() {
    super(EnchModule.LOGGER, "enchanting_stats", true, false);
  }
  
  protected void registerBuiltinCodecs() {
    registerDefaultCodec(Apotheosis.loc("enchanting_stats"), BlockStats.CODEC);
  }
  
  protected void beginReload() {
    super.beginReload();
    this.statsPerBlock.clear();
  }
  
  protected void onReload() {
    super.onReload();
    for (Iterator<BlockStats> iterator = this.registry.values().iterator(); iterator.hasNext(); ) {
      BlockStats bStats = iterator.next();
      bStats.blocks.forEach(b -> this.statsPerBlock.put(b, bStats.stats));
    } 
    computeAbsoluteMaxEterna();
  }
  
  public static float getEterna(BlockState state, Level world, BlockPos pos) {
    Block block = state.m_60734_();
    if (INSTANCE.statsPerBlock.containsKey(block))
      return ((Stats)INSTANCE.statsPerBlock.get(block)).eterna; 
    return state.getEnchantPowerBonus((LevelReader)world, pos);
  }
  
  public static float getMaxEterna(BlockState state, Level world, BlockPos pos) {
    Block block = state.m_60734_();
    if (INSTANCE.statsPerBlock.containsKey(block))
      return ((Stats)INSTANCE.statsPerBlock.get(block)).maxEterna; 
    return ((IEnchantingBlock)block).getMaxEnchantingPower(state, (LevelReader)world, pos);
  }
  
  public static float getQuanta(BlockState state, Level world, BlockPos pos) {
    Block block = state.m_60734_();
    if (INSTANCE.statsPerBlock.containsKey(block))
      return ((Stats)INSTANCE.statsPerBlock.get(block)).quanta; 
    return ((IEnchantingBlock)block).getQuantaBonus(state, (LevelReader)world, pos);
  }
  
  public static float getArcana(BlockState state, Level world, BlockPos pos) {
    Block block = state.m_60734_();
    if (INSTANCE.statsPerBlock.containsKey(block))
      return ((Stats)INSTANCE.statsPerBlock.get(block)).arcana; 
    return ((IEnchantingBlock)block).getArcanaBonus(state, (LevelReader)world, pos);
  }
  
  public static float getQuantaRectification(BlockState state, Level world, BlockPos pos) {
    Block block = state.m_60734_();
    if (INSTANCE.statsPerBlock.containsKey(block))
      return ((Stats)INSTANCE.statsPerBlock.get(block)).rectification; 
    return ((IEnchantingBlock)block).getQuantaRectification(state, (LevelReader)world, pos);
  }
  
  public static int getBonusClues(BlockState state, Level world, BlockPos pos) {
    Block block = state.m_60734_();
    if (INSTANCE.statsPerBlock.containsKey(block))
      return ((Stats)INSTANCE.statsPerBlock.get(block)).clues; 
    return ((IEnchantingBlock)block).getBonusClues(state, (LevelReader)world, pos);
  }
  
  public static float getAbsoluteMaxEterna() {
    return INSTANCE.absoluteMaxEterna;
  }
  
  private void computeAbsoluteMaxEterna() {
    this.absoluteMaxEterna = ((BlockStats)this.registry.values().stream().max(Comparator.comparingDouble(s -> s.stats.maxEterna)).get()).stats.maxEterna;
  }
}
